package org.apache.bcel.util;

import java.util.Map;
import org.apache.bcel.classfile.JavaClass;
import java.util.LinkedHashMap;

public class LruCacheClassPathRepository extends AbstractClassPathRepository
{
    private final LinkedHashMap<String, JavaClass> loadedClasses;
    
    public LruCacheClassPathRepository(final ClassPath path, final int cacheSize) {
        super(path);
        if (cacheSize < 1) {
            throw new IllegalArgumentException("cacheSize must be a positive number.");
        }
        final int initialCapacity = (int)(0.75 * cacheSize);
        final boolean accessOrder = true;
        this.loadedClasses = new LinkedHashMap<String, JavaClass>(initialCapacity, (float)cacheSize, true) {
            private static final long serialVersionUID = 1L;
            
            @Override
            protected boolean removeEldestEntry(final Map.Entry<String, JavaClass> eldest) {
                return this.size() > cacheSize;
            }
        };
    }
    
    @Override
    public void clear() {
        this.loadedClasses.clear();
    }
    
    @Override
    public JavaClass findClass(final String className) {
        return this.loadedClasses.get(className);
    }
    
    @Override
    public void removeClass(final JavaClass javaClass) {
        this.loadedClasses.remove(javaClass.getClassName());
    }
    
    @Override
    public void storeClass(final JavaClass javaClass) {
        this.loadedClasses.put(javaClass.getClassName(), javaClass);
        javaClass.setRepository(this);
    }
}
